@@ -4,12 +4,11 @@ use std::net::SocketAddr;
 use std::ops::Deref;
 use std::{collections::BTreeSet, marker::PhantomData, sync::Arc};
 
-use enum_map::EnumMap;
 use junction_api::backend::Backend;
 use junction_api::backend::BackendId;
 use junction_api::http::Route;
+use junction_api::Hostname;
 use smol_str::SmolStr;
-use xds_api::pb::google::protobuf;
 use xds_api::{
     pb::envoy::{
         config::{
@@ -17,6 +16,7 @@ use xds_api::{
             listener::v3 as xds_listener, route::v3 as xds_route,
         },
         extensions::filters::network::http_connection_manager::v3 as xds_http,
+        service::discovery::v3 as xds_discovery,
     },
     WellKnownTypes,
 };
@@ -100,34 +100,45 @@ impl_resource_version_from!(&String);
 impl_resource_version_from!(Arc<str>);
 impl_resource_version_from!(Box<str>);
 
+/// The type of an xDS resource we store in cache.
+///
+/// The order these are declared in is the xDS make-before-break order, so that
+/// the [enum_map] crate keeps enum maps in this order. This means any time we
+/// need to iterate an EnumMap's values, we're probably doing it in an order
+/// that keeps state updates sane.
 #[derive(Debug, Copy, Clone, PartialEq, Eq, enum_map::Enum, Hash, PartialOrd, Ord)]
 pub(crate) enum ResourceType {
-    Listener,
-    RouteConfiguration,
     Cluster,
     ClusterLoadAssignment,
+    Listener,
+    RouteConfiguration,
 }
 
 impl ResourceType {
     fn as_well_known(&self) -> WellKnownTypes {
         match self {
-            ResourceType::Listener => WellKnownTypes::Listener,
-            ResourceType::RouteConfiguration => WellKnownTypes::RouteConfiguration,
             ResourceType::Cluster => WellKnownTypes::Cluster,
             ResourceType::ClusterLoadAssignment => WellKnownTypes::ClusterLoadAssignment,
+            ResourceType::Listener => WellKnownTypes::Listener,
+            ResourceType::RouteConfiguration => WellKnownTypes::RouteConfiguration,
         }
     }
 
     fn from_well_known(wkt: WellKnownTypes) -> Option<Self> {
         match wkt {
-            WellKnownTypes::Listener => Some(Self::Listener),
-            WellKnownTypes::RouteConfiguration => Some(Self::RouteConfiguration),
             WellKnownTypes::Cluster => Some(Self::Cluster),
             WellKnownTypes::ClusterLoadAssignment => Some(Self::ClusterLoadAssignment),
+            WellKnownTypes::Listener => Some(Self::Listener),
+            WellKnownTypes::RouteConfiguration => Some(Self::RouteConfiguration),
             _ => None,
         }
     }
 
+    pub(crate) const fn supports_wildcard(&self) -> bool {
+        matches!(self, ResourceType::Cluster | ResourceType::Listener)
+    }
+
+    /// Return all of the known enum variants in xDS's make-before-break order.
     pub(crate) fn all() -> &'static [Self] {
         &[
             Self::Cluster,
@@ -146,42 +157,30 @@ impl ResourceType {
     }
 }
 
-#[derive(Debug)]
+#[derive(Clone, Debug)]
 pub(crate) enum ResourceVec {
-    Listener(Vec<xds_listener::Listener>),
-    RouteConfiguration(Vec<xds_route::RouteConfiguration>),
-    Cluster(Vec<xds_cluster::Cluster>),
-    ClusterLoadAssignment(Vec<xds_endpoint::ClusterLoadAssignment>),
+    Listener(VersionedVec<xds_listener::Listener>),
+    RouteConfiguration(VersionedVec<xds_route::RouteConfiguration>),
+    Cluster(VersionedVec<xds_cluster::Cluster>),
+    ClusterLoadAssignment(VersionedVec<xds_endpoint::ClusterLoadAssignment>),
 }
 
+type VersionedVec<T> = Vec<(ResourceVersion, T)>;
+
 impl ResourceVec {
-    pub(crate) fn from_any(
-        resource_type: ResourceType,
-        any: Vec<protobuf::Any>,
+    pub(crate) fn from_resources(
+        rtype: ResourceType,
+        resources: Vec<xds_discovery::Resource>,
     ) -> Result<Self, prost::DecodeError> {
-        match resource_type {
-            ResourceType::Listener => from_any_vec(any).map(Self::Listener),
-            ResourceType::RouteConfiguration => from_any_vec(any).map(Self::RouteConfiguration),
-            ResourceType::Cluster => from_any_vec(any).map(Self::Cluster),
+        match rtype {
+            ResourceType::Cluster => from_resource_vec(resources).map(Self::Cluster),
             ResourceType::ClusterLoadAssignment => {
-                from_any_vec(any).map(Self::ClusterLoadAssignment)
+                from_resource_vec(resources).map(Self::ClusterLoadAssignment)
+            }
+            ResourceType::Listener => from_resource_vec(resources).map(Self::Listener),
+            ResourceType::RouteConfiguration => {
+                from_resource_vec(resources).map(Self::RouteConfiguration)
             }
-        }
-    }
-
-    #[cfg(test)]
-    pub(crate) fn names(&self) -> Vec<String> {
-        macro_rules! clone_name {
-            ($v:expr, $name:ident) => {
-                $v.iter().map(|x| x.$name.clone()).collect()
-            };
-        }
-
-        match self {
-            ResourceVec::Listener(vec) => clone_name!(vec, name),
-            ResourceVec::RouteConfiguration(vec) => clone_name!(vec, name),
-            ResourceVec::Cluster(vec) => clone_name!(vec, name),
-            ResourceVec::ClusterLoadAssignment(vec) => clone_name!(vec, cluster_name),
         }
     }
 
@@ -194,45 +193,78 @@ impl ResourceVec {
             ResourceVec::ClusterLoadAssignment(_) => ResourceType::ClusterLoadAssignment,
         }
     }
+
+    #[cfg(test)]
+    pub(crate) fn to_resources(&self) -> Result<Vec<xds_discovery::Resource>, prost::EncodeError> {
+        match self {
+            ResourceVec::Listener(vec) => to_resource_vec(vec),
+            ResourceVec::RouteConfiguration(vec) => to_resource_vec(vec),
+            ResourceVec::Cluster(vec) => to_resource_vec(vec),
+            ResourceVec::ClusterLoadAssignment(vec) => to_resource_vec(vec),
+        }
+    }
 }
 
-fn from_any_vec<M: Default + prost::Name>(
-    any: Vec<protobuf::Any>,
-) -> Result<Vec<M>, prost::DecodeError> {
-    // TODO: if the type_url checks become a bottleneck, we can only check once
-    // and then decode every value instead of calling protobuf::Any::to_msg
-    let mut ms = Vec::with_capacity(any.len());
-    for a in any {
-        ms.push(a.to_msg()?);
+macro_rules! test_constructor {
+    ($name:ident, $variant:ident, $xds_type:ty) => {
+        impl ResourceVec {
+            #[cfg(test)]
+            pub(crate) fn $name<I: IntoIterator<Item = $xds_type>>(
+                version: ResourceVersion,
+                xs: I,
+            ) -> Self {
+                let data = xs.into_iter().map(|l| (version.clone(), l)).collect();
+                Self::$variant(data)
+            }
+        }
+    };
+}
+
+test_constructor!(from_listeners, Listener, xds_listener::Listener);
+test_constructor!(
+    from_route_configs,
+    RouteConfiguration,
+    xds_route::RouteConfiguration
+);
+test_constructor!(from_clusters, Cluster, xds_cluster::Cluster);
+test_constructor!(
+    from_load_assignments,
+    ClusterLoadAssignment,
+    xds_endpoint::ClusterLoadAssignment
+);
+
+fn from_resource_vec<M: Default + prost::Name>(
+    resources: Vec<xds_discovery::Resource>,
+) -> Result<VersionedVec<M>, prost::DecodeError> {
+    let mut ms = Vec::with_capacity(resources.len());
+    for r in resources {
+        let Some(any) = r.resource else {
+            continue;
+        };
+        ms.push((ResourceVersion::from(r.version), any.to_msg()?));
     }
 
     Ok(ms)
 }
 
-/// A specialized set of `ResourceType`s.
-#[derive(Clone, Debug, Default)]
-pub(crate) struct ResourceTypeSet(EnumMap<ResourceType, bool>);
+#[cfg(test)]
+fn to_resource_vec<M: prost::Name>(
+    xs: &VersionedVec<M>,
+) -> Result<Vec<xds_discovery::Resource>, prost::EncodeError> {
+    use xds_api::pb::google::protobuf;
 
-impl ResourceTypeSet {
-    pub(crate) fn len(&self) -> usize {
-        self.0.values().filter(|x| **x).count()
+    let mut resources = Vec::with_capacity(xs.len());
+
+    for (v, msg) in xs {
+        let as_any = protobuf::Any::from_msg(msg)?;
+        resources.push(xds_discovery::Resource {
+            resource: Some(as_any),
+            version: v.to_string(),
+            ..Default::default()
+        })
     }
 
-    pub(crate) fn values(&self) -> impl Iterator<Item = ResourceType> {
-        self.0.into_iter().filter_map(|(k, v)| v.then_some(k))
-    }
-
-    pub(crate) fn is_empty(&self) -> bool {
-        !self.0.values().any(|e| *e)
-    }
-
-    pub(crate) fn insert(&mut self, resource_type: ResourceType) {
-        self.0[resource_type] = true
-    }
-
-    pub(crate) fn contains(&self, resource_type: ResourceType) -> bool {
-        self.0[resource_type]
-    }
+    Ok(resources)
 }
 
 /// A typed reference to another resource.
@@ -302,7 +334,7 @@ pub(crate) enum ApiListenerData {
     Inlined(RouteConfigData),
 }
 
-fn api_listener(
+fn http_connection_manager(
     listener: &xds_listener::Listener,
 ) -> Result<xds_http::HttpConnectionManager, ResourceError> {
     let api_listener = listener
@@ -322,7 +354,7 @@ impl ApiListener {
     pub(crate) fn from_xds(name: &str, xds: xds_listener::Listener) -> Result<Self, ResourceError> {
         use xds_http::http_connection_manager::RouteSpecifier;
 
-        let conn_manager = api_listener(&xds)?;
+        let conn_manager = http_connection_manager(&xds)?;
         let data = match &conn_manager.route_specifier {
             Some(RouteSpecifier::Rds(rds)) => {
                 let name = rds.route_config_name.clone();
@@ -448,6 +480,14 @@ pub(crate) struct Cluster {
 }
 
 impl Cluster {
+    pub(crate) fn dns_name(&self) -> Option<(Hostname, u16)> {
+        let id = &self.backend_lb.config.id;
+        match &id.service {
+            junction_api::Service::Dns(dns) => Some((dns.hostname.clone(), id.port)),
+            _ => None,
+        }
+    }
+
     pub(crate) fn from_xds(
         xds: xds_cluster::Cluster,
         default_action: Option<&xds_route::RouteAction>,
